searchData={"items":[{"type":"behaviour","title":"WebSockex","doc":"A client handles negotiating the connection, then sending frames, receiving\nframes, closing, and reconnecting that connection.\n\nA simple client implementation would be:\n\n```\ndefmodule WsClient do\n  use WebSockex\n\n  def start_link(url, state) do\n    WebSockex.start_link(url, __MODULE__, state)\n  end\n\n  def handle_frame({:text, msg}, state) do\n    IO.puts \"Received a message: #{msg}\"\n    {:ok, state}\n  end\n\n  def handle_cast({:send, {type, msg} = frame}, state) do\n    IO.puts \"Sending #{type} frame with payload: #{msg}\"\n    {:reply, frame, state}\n  end\nend\n```","ref":"WebSockex.html"},{"type":"behaviour","title":"Supervision - WebSockex","doc":"WebSockex is implemented as an OTP Special Process and as a result will fit\ninto supervision trees.\n\nWebSockex also supports the Supervisor children format introduced in Elixir\n1.5. Meaning that a child specification could be `{ClientModule, [state]}`.\n\nHowever, since there is a possibility that you would like to provide a\n`t:WebSockex.Conn/0` or a url as well as the state, there are two versions of\nthe `child_spec` function. If you need functionality beyond that it is\nrecommended that you override the function or define your own.\n\nJust remember to use the version that corresponds with your `start_link`'s\narity.","ref":"WebSockex.html#module-supervision"},{"type":"function","title":"WebSockex.cast/2","doc":"Asynchronously sends a message to a client that is handled by `c:handle_cast/2`.","ref":"WebSockex.html#cast/2"},{"type":"type","title":"WebSockex.client/0","doc":"","ref":"WebSockex.html#t:client/0"},{"type":"type","title":"WebSockex.close_code/0","doc":"An integer between 1000 and 4999 that specifies the reason for closing the connection.","ref":"WebSockex.html#t:close_code/0"},{"type":"type","title":"WebSockex.close_error/0","doc":"The error returned when a connection fails to be established.","ref":"WebSockex.html#t:close_error/0"},{"type":"type","title":"WebSockex.close_frame/0","doc":"The frame sent when the negotiating a connection closure.","ref":"WebSockex.html#t:close_frame/0"},{"type":"type","title":"WebSockex.close_reason/0","doc":"The reason a connection was closed.\n\nA `:normal` reason is the same as a `1000` reason with no payload.\n\nIf the peer closes the connection abruptly without a close frame then the\nclose reason is `{:remote, :closed}`.","ref":"WebSockex.html#t:close_reason/0"},{"type":"callback","title":"WebSockex.code_change/3","doc":"Invoked when a new version the module is loaded during runtime.","ref":"WebSockex.html#c:code_change/3"},{"type":"type","title":"WebSockex.connection_status_map/0","doc":"A map that contains information about the failure to connect.\n\nThis map contains the error, attempt number, and the `t:WebSockex.Conn.t/0`\nthat was used to attempt the connection.","ref":"WebSockex.html#t:connection_status_map/0"},{"type":"type","title":"WebSockex.debug_opts/0","doc":"Debug options to be parsed by `:sys.debug_options/1`.\n\nThese options can also be set after the process is running using the functions in\nthe Erlang `:sys` module.","ref":"WebSockex.html#t:debug_opts/0"},{"type":"callback","title":"WebSockex.format_status/2","doc":"Invoked to retrieve a formatted status of the state in a WebSockex process.\n\nThis optional callback is used when you want to edit the values returned when\ninvoking `:sys.get_status`.\n\nThe second argument is a two-element list with the order of `[pdict, state]`.","ref":"WebSockex.html#c:format_status/2"},{"type":"type","title":"WebSockex.frame/0","doc":"","ref":"WebSockex.html#t:frame/0"},{"type":"callback","title":"WebSockex.handle_cast/2","doc":"Invoked to handle asynchronous `cast/2` messages.","ref":"WebSockex.html#c:handle_cast/2"},{"type":"callback","title":"WebSockex.handle_connect/2","doc":"Invoked after a connection is established.\n\nThis is invoked after both the initial connection and a reconnect.","ref":"WebSockex.html#c:handle_connect/2"},{"type":"callback","title":"WebSockex.handle_disconnect/2","doc":"Invoked when the WebSocket disconnects from the server.\n\nThis callback is only invoked in the event of a connection failure. In cases\nof crashes or other errors the process will terminate immediately\nskipping this callback.\n\nIf the `handle_initial_conn_failure: true` option is provided during process\nstartup, then this callback will be invoked if the process fails to establish\nan initial connection.\n\nIf a connection is established by reconnecting, the `c:handle_connect/2`\ncallback will be invoked.\n\nThe possible returns for this callback are:\n\n- `{:ok, state}` will continue the process termination.\n- `{:reconnect, state}` will attempt to reconnect instead of terminating.\n- `{:reconnect, conn, state}` will attempt to reconnect with the connection\n  data in `conn`. `conn` is expected to be a `t:WebSockex.Conn.t/0`.","ref":"WebSockex.html#c:handle_disconnect/2"},{"type":"callback","title":"WebSockex.handle_frame/2","doc":"Invoked on the reception of a frame on the socket.\n\nThe control frames have possible payloads, when they don't have a payload\nthen the frame will have `nil` as the payload. e.g. `{:ping, nil}`","ref":"WebSockex.html#c:handle_frame/2"},{"type":"callback","title":"WebSockex.handle_info/2","doc":"Invoked to handle all other non-WebSocket messages.","ref":"WebSockex.html#c:handle_info/2"},{"type":"callback","title":"WebSockex.handle_ping/2","doc":"Invoked when the Websocket receives a ping frame","ref":"WebSockex.html#c:handle_ping/2"},{"type":"callback","title":"WebSockex.handle_pong/2","doc":"Invoked when the Websocket receives a pong frame.","ref":"WebSockex.html#c:handle_pong/2"},{"type":"function","title":"WebSockex.handle_terminate_close/4","doc":"","ref":"WebSockex.html#handle_terminate_close/4"},{"type":"function","title":"WebSockex.init/6","doc":"","ref":"WebSockex.html#init/6"},{"type":"type","title":"WebSockex.option/0","doc":"Options values for `start_link`.\n\n- `:async` - Replies with `{:ok, pid}` before establishing the connection.\n  This is useful for when attempting to connect indefinitely, this way the\n  process doesn't block trying to establish a connection.\n- `:handle_initial_conn_failure` - When set to `true` a connection failure\n  while establishing the initial connection won't immediately return an error\n  and instead will invoke the `c:handle_disconnect/2` callback. This option\n  only matters during process initialization. The `handle_disconnect`\n  callback is always invoked if an established connection is lost.\n- `:debug` - Options to set the debug options for `:sys.handle_debug`.\n- `:name` - An atom that the registers the process with name locally.\n  Can also be a `{:via, module, term}` or `{:global, term}` tuple.\n\nOther possible option values include: `t:WebSockex.Conn.connection_option/0`","ref":"WebSockex.html#t:option/0"},{"type":"type","title":"WebSockex.options/0","doc":"","ref":"WebSockex.html#t:options/0"},{"type":"function","title":"WebSockex.send_frame/2","doc":"Sends a frame through the WebSocket.\n\nIf the connection is either connecting or closing then this will return an\nerror tuple with a `WebSockex.NotConnectedError` exception struct as the\nsecond element.\n\nIf a connection failure is discovered while sending then it will return an\nerror tuple with a `WebSockex.ConnError` exception struct as the second\nelement.","ref":"WebSockex.html#send_frame/2"},{"type":"function","title":"WebSockex.start/4","doc":"Starts a `WebSockex` process.\n\nActs like `start_link/4`, except doesn't link the current process.\n\nSee `start_link/4` for more information.","ref":"WebSockex.html#start/4"},{"type":"function","title":"WebSockex.start_link/4","doc":"Starts a `WebSockex` process linked to the current process.\n\nFor available option values see `t:option/0`.\n\nIf a `WebSockex.Conn.t` is used in place of a url string, then the options\navailable in `t:WebSockex.Conn.connection_option/0` have effect.\n\nThe callback `c:handle_connect/2` is invoked after the connection is\nestablished.","ref":"WebSockex.html#start_link/4"},{"type":"callback","title":"WebSockex.terminate/2","doc":"Invoked when the process is terminating.","ref":"WebSockex.html#c:terminate/2"},{"type":"module","title":"WebSockex.Application","doc":"","ref":"WebSockex.Application.html"},{"type":"function","title":"WebSockex.Application.start/2","doc":"Sets the `ws` and `wss` default ports for the `URI` module.","ref":"WebSockex.Application.html#start/2"},{"type":"module","title":"WebSockex.Conn","doc":"Handles establishing and controlling the TCP connection.\n\nDispatches to the correct module for the underlying connection. (`:gen_tcp` or `:ssl`)\n\nIs woefully inadequite for verifying proper peers in SSL connections.","ref":"WebSockex.Conn.html"},{"type":"function","title":"WebSockex.Conn.build_request/2","doc":"Builds the request to be sent along the newly opened socket.\n\nThe key parameter is part of the websocket handshake process.","ref":"WebSockex.Conn.html#build_request/2"},{"type":"type","title":"WebSockex.Conn.certification/0","doc":"","ref":"WebSockex.Conn.html#t:certification/0"},{"type":"function","title":"WebSockex.Conn.close_socket/1","doc":"Closes the socket and returns the Conn map without the socket.\n\nWhen the `:socket` field is `nil` in the struct, the function just returns\nthe struct as is.","ref":"WebSockex.Conn.html#close_socket/1"},{"type":"type","title":"WebSockex.Conn.connection_option/0","doc":"Options used when establishing a tcp or ssl connection.\n\n- `:extra_headers` - defines other headers to be send in the opening request.\n- `:insecure` - Determines whether to verify the peer in a SSL connection.\n  SSL peer verification is currenctly broken and only works in certain cases\n  in which the `:cacerts` are also provided. Sorry. _Defaults to `true`_.\n- `:cacerts` - The CA certifications for use in an secure connection when the\n  `:insecure` option is `false` (has no effect when `:insecure is true`).\n  These certifications need a list of decoded binaries. See the\n  [Erlang `:public_key` module][public_key] for more information.\n- `:socket_connect_timeout` - Timeout in ms for creating a TCP connection,\n  default 6000 ms.\n- `:socket_recv_timeout` - Timeout in ms for receiving a HTTP response header\n  from socket, default 5000 ms.\n- `:ssl_options` - extra options for an SSL connection\n- `:socket_options` - extra options for the TCP part of the connection\n\n[public_key]: http://erlang.org/doc/apps/public_key/using_public_key.html","ref":"WebSockex.Conn.html#t:connection_option/0"},{"type":"function","title":"WebSockex.Conn.controlling_process/2","doc":"Set the socket's controlling process.","ref":"WebSockex.Conn.html#controlling_process/2"},{"type":"function","title":"WebSockex.Conn.handle_response/2","doc":"Waits for the request response, decodes the packet, and returns the response\nheaders.\n\nSends any access information in the buffer back to the process as a message\nto be processed.","ref":"WebSockex.Conn.html#handle_response/2"},{"type":"type","title":"WebSockex.Conn.header/0","doc":"","ref":"WebSockex.Conn.html#t:header/0"},{"type":"function","title":"WebSockex.Conn.new/2","doc":"Returns a new `WebSockex.Conn` struct from a uri and options.","ref":"WebSockex.Conn.html#new/2"},{"type":"function","title":"WebSockex.Conn.open_socket/1","doc":"Opens a socket to a uri and returns a conn struct.","ref":"WebSockex.Conn.html#open_socket/1"},{"type":"function","title":"WebSockex.Conn.parse_host/1","doc":"Parses a URI host\nHost can be \"x.y.z.t\" or \"some.name.domain\". If \"x.y.z.t\", the function\nwill return a valid :inet.ip_address() which __MODULE__.open_socket\naccepts. This will prevent extra DNS operations which can time out\nin some contexts","ref":"WebSockex.Conn.html#parse_host/1"},{"type":"function","title":"WebSockex.Conn.parse_url/1","doc":"Parses a url string for a valid URI","ref":"WebSockex.Conn.html#parse_url/1"},{"type":"function","title":"WebSockex.Conn.set_active/2","doc":"Sets the socket to active.","ref":"WebSockex.Conn.html#set_active/2"},{"type":"type","title":"WebSockex.Conn.socket/0","doc":"","ref":"WebSockex.Conn.html#t:socket/0"},{"type":"function","title":"WebSockex.Conn.socket_send/2","doc":"Sends data using the `conn_mod` module.","ref":"WebSockex.Conn.html#socket_send/2"},{"type":"type","title":"WebSockex.Conn.t/0","doc":"","ref":"WebSockex.Conn.html#t:t/0"},{"type":"type","title":"WebSockex.Conn.transport/0","doc":"","ref":"WebSockex.Conn.html#t:transport/0"},{"type":"module","title":"WebSockex.Frame","doc":"Functions for parsing and encoding frames.","ref":"WebSockex.Frame.html"},{"type":"type","title":"WebSockex.Frame.buffer/0","doc":"The incomplete or unhandled remainder of a binary","ref":"WebSockex.Frame.html#t:buffer/0"},{"type":"type","title":"WebSockex.Frame.close_code/0","doc":"","ref":"WebSockex.Frame.html#t:close_code/0"},{"type":"function","title":"WebSockex.Frame.encode_frame/1","doc":"Encodes a frame into a binary for sending.","ref":"WebSockex.Frame.html#encode_frame/1"},{"type":"type","title":"WebSockex.Frame.frame/0","doc":"","ref":"WebSockex.Frame.html#t:frame/0"},{"type":"type","title":"WebSockex.Frame.opcode/0","doc":"","ref":"WebSockex.Frame.html#t:opcode/0"},{"type":"function","title":"WebSockex.Frame.parse_fragment/2","doc":"Parses and combines two frames in a fragmented segment.","ref":"WebSockex.Frame.html#parse_fragment/2"},{"type":"function","title":"WebSockex.Frame.parse_frame/1","doc":"Parses a bitstring and returns a frame.","ref":"WebSockex.Frame.html#parse_frame/1"},{"type":"type","title":"WebSockex.Frame.utf8/0","doc":"This is required to be valid UTF-8","ref":"WebSockex.Frame.html#t:utf8/0"},{"type":"exception","title":"WebSockex.CallingSelfError","doc":"","ref":"WebSockex.CallingSelfError.html"},{"type":"function","title":"WebSockex.CallingSelfError.message/1","doc":"","ref":"WebSockex.CallingSelfError.html#message/1"},{"type":"exception","title":"WebSockex.FragmentParseError","doc":"","ref":"WebSockex.FragmentParseError.html"},{"type":"function","title":"WebSockex.FragmentParseError.message/1","doc":"","ref":"WebSockex.FragmentParseError.html#message/1"},{"type":"exception","title":"WebSockex.FrameEncodeError","doc":"","ref":"WebSockex.FrameEncodeError.html"},{"type":"function","title":"WebSockex.FrameEncodeError.message/1","doc":"","ref":"WebSockex.FrameEncodeError.html#message/1"},{"type":"exception","title":"WebSockex.FrameError","doc":"","ref":"WebSockex.FrameError.html"},{"type":"function","title":"WebSockex.FrameError.message/1","doc":"","ref":"WebSockex.FrameError.html#message/1"},{"type":"exception","title":"WebSockex.InvalidFrameError","doc":"","ref":"WebSockex.InvalidFrameError.html"},{"type":"function","title":"WebSockex.InvalidFrameError.message/1","doc":"","ref":"WebSockex.InvalidFrameError.html#message/1"},{"type":"exception","title":"WebSockex.NotConnectedError","doc":"","ref":"WebSockex.NotConnectedError.html"},{"type":"function","title":"WebSockex.NotConnectedError.message/1","doc":"","ref":"WebSockex.NotConnectedError.html#message/1"},{"type":"exception","title":"WebSockex.RequestError","doc":"","ref":"WebSockex.RequestError.html"},{"type":"extras","title":"WebSockex","doc":"# WebSockex\n\n![Build](https://github.com/dominicletz/websockex/actions/workflows/ci.yml/badge.svg)\n\nAn Elixir Websocket Client.\n\nA simple implementation could be\n\n```elixir\ndefmodule WebSocketExample do\n  use WebSockex\n\n  def start_link(url, state) do\n    WebSockex.start_link(url, __MODULE__, state)\n  end\n\n  def handle_frame({type, msg}, state) do\n    IO.puts \"Received Message - Type: #{inspect type} -- Message: #{inspect msg}\"\n    {:ok, state}\n  end\n\n  def handle_cast({:send, {type, msg} = frame}, state) do\n    IO.puts \"Sending #{type} frame with payload: #{msg}\"\n    {:reply, frame, state}\n  end\nend\n```\n\nSee the `examples/` directory for other examples or take a look at the [documentation][docs].","ref":"readme.html"},{"type":"extras","title":"Installation - WebSockex","doc":"Add `websockex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:websockex_wt, \"~> 0.4.4\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"With Elixir releases prior to version  1.4 - WebSockex","doc":"Ensure `websockex` is started before your application:\n\n```elixir\ndef application do\n  [applications: [:websockex]]\nend\n```","ref":"readme.html#with-elixir-releases-prior-to-version-1-4"},{"type":"extras","title":"Why WebSockex? - WebSockex","doc":"WebSockex was conceived after trying other libraries and realizing that I needed something tested, that actually\nimplemented the spec, provided information about the connection, and could fit into a supervision tree. There was\nnothing that really fit into all those categories, so WebSockex was created.\n\nThere are other libraries out there can fit into some of the categories, but I consider WebSockex the best option if\nyou want a callback inspired approach where most of the protocol workings are abstracted out of your way.\n\nIf you are afraid that WebSockex isn't stable enough or have some other problem with it that you don't feel like\ntelling me about, then I would suggest the excellent [`gun` library][gun_hex]. It's a bit harder to use, and requires\nsome knowledge of the spec. However it is an excellent library.\n\n[gun_hex]: https://hex.pm/packages/gun","ref":"readme.html#why-websockex"},{"type":"extras","title":"Supervision and Linking - WebSockex","doc":"A WebSockex based process is a easily able to fit into any supervision tree. It supports all the necessary capabilites\nto do so. In addition, it supports the `Supervisor` children format introduced in Elixir 1.5. So in any version of\nElixir after 1.5, you can simply do:\n\n```elixir\ndefmodule MyApp.Client do\n  use WebSockex\n\n  def start_link(state) do\n    WebSockex.start_link(\"ws://myapp.ninja\", __MODULE__, state)\n  end\nend\n\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {MyApp.Client, [\"WebSockex is Great\"]}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\nSee the Supervision section in the `WebSockex` module documentation for more details on how this works.\n\nWebSockex also supports both linked(`start_link/3`) and unlinked(`start/3`) processes.\n\n```elixir\niex> {:ok, pid} = WebSockex.start_link(url, __MODULE__, state)\niex> {:ok, pid} = WebSockex.start(url, __MODULE__, state)\n```\n\nHowever, the recommendation is to always use `start_link/3` and if necessary trap exits.\n\nThis is because `start/3` creates a detached process and has the capability to produce zombie processes outside of any\napplication tree. This is generally a good piece of advice for any process, however since a module using WebSockex\nbevhaviour can be written as a self-sustaining tcp connection. I feel like it is even more important to express this\nparticular piece of advice here.","ref":"readme.html#supervision-and-linking"},{"type":"extras","title":"Telemetry - WebSockex","doc":"Websockex clients emit the following telemetry events:\n\n* `[:websockex, :connected]`\n* `[:websockex, :disconnected]`\n* `[:websockex, :frame, :received]`\n* `[:websockex, :frame, :sent]`\n* `[:websockex, :terminate]`\n\nFor all these events, the measurements is `%{time: System.system_time()}` and they all share common metadata as a map containing the `:conn` and the `:module` keys. For frame events, the metadata also contains the `:frame` key. For disconnections and terminations, it will contain the `:reason` key.","ref":"readme.html#telemetry"},{"type":"extras","title":"Tips - WebSockex","doc":"","ref":"readme.html#tips"},{"type":"extras","title":"Terminating with :normal after an Exceptional Close or Error - WebSockex","doc":"Usually you'll want to negotiate and handle any abnormal close event or error leading to it, as per WS Spec, but there might be cases where you simply want the socket to exit as if it was a normal event, even if it was abruptly closed or another exception was raised. In those cases you can define the terminate callback and return `exit(:normal)` from it.\n```elixir\ndef terminate(reason, state) do\n    IO.puts(\"\\nSocket Terminating:\\n#{inspect reason}\\n\\n#{inspect state}\\n\")\n    exit(:normal)\nend\n```","ref":"readme.html#terminating-with-normal-after-an-exceptional-close-or-error"},{"type":"extras","title":"Debugging - WebSockex","doc":"WebSockex supports the debugging mechanism for [OTP Special Processes][special_process] provided through the `:sys` module.\n\nSince WebSockex rolls its own Special Process implementation, it's able to provide a lot more information than a regular\n`GenServer`.\n\nIf, for example, I enable tracing with `EchoClient` from the examples (with `Logger` off), I would get this:\n\n```elixir\niex> {:ok, pid} = EchoClient.start_link(debug: [:trace])\n*DBG* #PID<0.371.0> attempting to connect\n*DBG* #PID<0.371.0> sucessfully connected\n{:ok, #PID<0.371.0>}\niex> EchoClient.echo(pid, \"Hello\")\n*DBG* #PID<0.371.0> sending frame: {:text, \"Hello\"}\n:ok\n*DBG* #PID<0.371.0> received frame: {:text, \"Hello\"}\n*DBG* #PID<0.371.0> received frame: :ping\n*DBG* #PID<0.371.0> replying from :handle_ping with :pong\niex> EchoClient.echo(pid, \"Close the things!\")\n*DBG* #PID<0.371.0> sending frame: {:text, \"Close the things!\"}\n:ok\n*DBG* #PID<0.371.0> received frame: {:text, \"Close the things!\"}\n*DBG* #PID<0.371.0> closing with local reason: {:local, :normal}\n*DBG* #PID<0.371.0> sending close frame: {:local, :normal}\n*DBG* #PID<0.371.0> forcefully closed the connection because the server was taking too long close\n```\n\nYou could also enable tracing after a process has started like this:\n\n```elixir\niex> {:ok, pid} = EchoClient.start_link()\niex> :sys.trace(pid, true)\n:ok\niex> EchoClient.echo(pid, \"Hi\")\n*DBG* #PID<0.379.0> sending frame: {:text, \"Hi\"}\n:ok\n*DBG* #PID<0.379.0> received frame: {:text, \"Hi\"}\n```\n\n[special_process]: http://erlang.org/doc/design_principles/spec_proc.html\n[docs]: https://hexdocs.pm/websockex","ref":"readme.html#debugging"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}